const CURRENT_LOCALE = chrome.i18n.getMessage('@@ui_locale');
const MULTIPART_FORMDATA_BOUNDARY = 'Google_Chrome_Screen_Capture';
const HIDE_ERROR_INFO_DELAY_TIME = 5000;
const TPU_PORTAL_URL = 'http://portal.tpu.ru/f_idotpu/api/settings.json';
var $j = jQuery.noConflict();

var UploadUI = {
  currentSite: '',
  uploading: false,
  sites: {},

  registerSite: function(id, siteObject, p) {
    this.sites[id] = {obj: siteObject, param: p};
  },

  getSiteObject: function(id) {
    return this.sites[id].obj;
  },

  getSiteObjectParam: function(id) {
    return this.sites[id].param;
  },
  
  setUploading: function(state) {
    UploadUI.uploading = state;
  },

  getSites: function(callback) {
	  $j.get(
	      TPU_PORTAL_URL, 
	      function(data) {
	    	  if (callback)
	    		  callback('success', data);
		  
	  },'json')
	  .error(function() {
    	  if (callback)
    		  callback('error', 'service_unavailable');
	  });  
  },
  
  init: function() {  
    // Import style sheet for current locale
    UI.addStyleSheet('./i18n_styles/en_US_upload_image.css');

    // Get i18n message
    i18nReplace('lastStep', 'return_to_site_selection');
    i18nReplace('closeUploadWrapper', 'close_upload_wrapper');
   
    // Add event listeners
    $('btnUpload').addEventListener('click', UploadUI.showUploadWrapper, false);    
    $('closeUploadWrapper').addEventListener('click', UploadUI.hideUploadWrapper, false);
    $('lastStep').addEventListener('click', UploadUI.showUploadSitesWrapper, false);    
    $('shareImage').addEventListener('click', UploadUI.shareImage, false);    
  },
  
  showUploadWrapper: function() {
	var uploadWrapper = $('uploadWrapper');
	UI.show(uploadWrapper);
	
	// Reset upload wrapper position
	var viewportWidth = window.innerWidth;
	var viewportHeight = window.innerHeight;
	var wrapperWidth = uploadWrapper.offsetWidth;
	var wrapperHeight = uploadWrapper.offsetHeight;
	
	var left = (viewportWidth - wrapperWidth) / 2;
	var top = (viewportHeight - wrapperHeight) / 3;
	left = left < 0 ? 0 : left;
	top = top < 0 ? 0 : top;
	
	var scrollTop = document.body.scrollTop;
	var scrollLeft = document.body.scrollLeft;
	
	UI.setStyle(uploadWrapper, {
	  top: top + scrollTop + 'px',
	  left: left + scrollLeft + 'px'
	});
	UI.hide($('shareImage'));
	UploadUI.showUploadSitesWrapper();
	UploadUI.showOverlay();	
  },

  hideUploadWrapper: function() {
	UI.hide($('uploadWrapper'));
	UploadUI.hideOverlay();
  },
  
  showOverlay: function() {
    var overlay = $('overlay');
    UI.setStyle(overlay, {
      width: document.body.scrollWidth + 'px',
      height: document.body.scrollHeight + 'px'
    });
    UI.show($('overlay'));
  },

  hideOverlay: function() {
    UI.hide($('overlay'));
  },
	  
  showUploadSitesWrapper: function() {
    $j('#loadDivSite').show();
    $j('#loadingSites').text( chrome.i18n.getMessage('oading') );
    $j('#uploadHeaderTitle').text( chrome.i18n.getMessage('share') );
    
    var callback = function(result, response) {
        if (result == 'success') {        	
        	$j('#uploadContentWrapperSites').empty();
        	$j.each(response, function(i, el) {        		
            	var btn = $j('<span class="btn shareBtnTo" siteId="' + el.site + '">'+
    					'<span class="tpuSite">'+
    						'<img class="logo" src="images/tpu_logo.png" alt="' + el.title + '">'+
    						'<span class="logo_span" id="shareTo' + el.site + '"">' + el.title + '</span>'+
    				    '</span>'+
    				 '</span>');    	
            	UploadUI.registerSite(el.site, TPU, el);
            	$j('#uploadContentWrapperSites').append(btn);        		
        	});
        	$j('#uploadContentWrapperSites').show();
        	
            $j('.shareBtnTo').click(function() {
            	var siteId = $j(this).attr('siteId');
            	UploadUI.showUploadContentWrapper(siteId);
            });        	
        }
        else {
        	$j('#uploadContentWrapperSites').html(chrome.i18n.getMessage(response));
        } // if
    	$j('#loadDivSite').hide();
    }
    UI.show($('uploadSitesWrapper'));    
    UploadUI.hideUploadContentWrapper();
    UploadUI.getSites(callback);
    UI.hide($('lastStep'));
    UI.hide($('shareImage'));
  },

  hideUploadSitesWrapper: function() {
    UI.hide($('uploadSitesWrapper'));
  },
  
  hideUploadContentWrapper: function() {
    UI.hide($('uploadContentWrapper'));
  },
  
  /**
   * форма конкретного сайта
   */
  showUploadContentWrapper: function(site) {
    UploadUI.currentSite = site;

    var p = UploadUI.getSiteObjectParam(site);
    
    $j('#uploadHeaderTitle').text( chrome.i18n.getMessage('share_to') + ' "' + p.title + '"' );
    
    // Update upload wrapper UI
    UploadUI.hideUploadSitesWrapper();
    UI.show($('uploadContentWrapper'));
    UI.show($('lastStep'));

    var siteobj = UploadUI.getSiteObject(site);
    
    $j('#capturing').text( chrome.i18n.getMessage('loading') );
    
    $j('#loadDiv').show();
    var callback = function(result, response) {
        if (result == 'success') {
        	$j('#uploadContentWrapperBody').html(response.html);           	
        	UI.show($('shareImage'));
        }
        else {
        	$j('#uploadContentWrapperBody').html(chrome.i18n.getMessage('failed_to_connect_to_server'));        	
        }
    	$j('#loadDiv').hide();
    }
    siteobj.getForm(callback, p.ajax_url);
  },
  
  shareImage: function() {
    var siteobj = UploadUI.getSiteObject(UploadUI.currentSite);
    var p = UploadUI.getSiteObjectParam(UploadUI.currentSite);
    
    var callback = function(result, response) {
    	if (result =='begin' ) {
    	    $j('.progress').show();    		
    	}
    	if (result =='success' ) {
        	$j('#uploadContentWrapperBody')
	    		.hide()
	    		.html('<div class="done_msg">' + p.success_msg + '</div>')
	    		.fadeIn('slow');
            $j('#uploadHeaderTitle').text( chrome.i18n.getMessage('send_ok') );
    		UI.hide($('shareImage'));
    		UI.hide($('lastStep'));
    		$j('.progress')
    			.fadeOut('slow');
    	}
    	if (result =='failure' ) {
        	$j('.progress').hide();    		
            $j('#uploadHeaderTitle').text( chrome.i18n.getMessage(response) );
        	$j('#uploadContentWrapperBody')
	    		.fadeOut('slow')
	    		.html('<div class="done_msg">' + chrome.i18n.getMessage(response) + '</div>')
	    		.fadeIn('slow');
    	}
    }
    var callback_upload = function(percentVal) {
        $j('#uploadHeaderTitle').text( chrome.i18n.getMessage('progress_info') );

        $j('.bar').width(percentVal);
        $j('.percent').html(percentVal);	            	
    }
    
  	siteobj.upload(UploadUI.getImageData(), callback, callback_upload, p.upload_url);	  
  },
  
  getImageData: function() {
	if (photoshop)
		photoshop.draw();
    var dataUrl = $('canvas').toDataURL('image/png');
    var imageDataIndex = dataUrl.indexOf('data:image/png;base64,');
    if (imageDataIndex != 0) {
      return;
    }

    // Decode to binary data
    return atob(dataUrl.substr(imageDataIndex + 22));
  }
};